@@ -18,7 +18,7 @@
 #include <hydra.h>
 #include <daemon.h>
 #include <threading/rwlock.h>
-#include <utils/linked_list.h>
+#include <collections/linked_list.h>
 
 
 typedef struct private_shunt_manager_t private_shunt_manager_t;
@@ -206,6 +206,7 @@ METHOD(shunt_manager_t, uninstall, bool,
 		return FALSE;
 	}
 	uninstall_shunt_policy(child);
+	child->destroy(child);
 	return TRUE;
 }
 
