@@ -260,6 +260,11 @@ struct task_manager_t {
 	enumerator_t* (*create_task_enumerator)(task_manager_t *this,
 											task_queue_t queue);
 
+	/**
+	 * Flush all tasks, regardless of the queue.
+	 */
+	void (*flush)(task_manager_t *this);
+
 	/**
 	 * Flush a queue, cancelling all tasks.
 	 *
