@@ -70,8 +70,10 @@ enum integrity_algorithm_t {
 	AUTH_HMAC_SHA2_256_256 = 1027,
 	/** SHA384 full length truncation variant, as used in TLS */
 	AUTH_HMAC_SHA2_384_384 = 1028,
+	/** SHA512 full length truncation variant */
+	AUTH_HMAC_SHA2_512_512 = 1029,
 	/** draft-kanno-ipsecme-camellia-xcbc, not yet assigned by IANA */
-	AUTH_CAMELLIA_XCBC_96 = 1029,
+	AUTH_CAMELLIA_XCBC_96 = 1030,
 };
 
 /**
@@ -91,8 +93,10 @@ struct signer_t {
 	 *
 	 * @param data		a chunk containing the data to sign
 	 * @param buffer	pointer where the signature will be written
+	 * @return			TRUE if signature created successfully
 	 */
-	void (*get_signature) (signer_t *this, chunk_t data, u_int8_t *buffer);
+	bool (*get_signature)(signer_t *this, chunk_t data,
+						  u_int8_t *buffer) __attribute__((warn_unused_result));
 
 	/**
 	 * Generate a signature and allocate space for it.
@@ -102,8 +106,10 @@ struct signer_t {
 	 *
 	 * @param data		a chunk containing the data to sign
 	 * @param chunk		chunk which will hold the allocated signature
+	 * @return			TRUE if signature allocated successfully
 	 */
-	void (*allocate_signature) (signer_t *this, chunk_t data, chunk_t *chunk);
+	bool (*allocate_signature)(signer_t *this, chunk_t data,
+						  chunk_t *chunk) __attribute__((warn_unused_result));
 
 	/**
 	 * Verify a signature.
@@ -116,33 +122,35 @@ struct signer_t {
 	 * @param signature	a chunk containing the signature
 	 * @return			TRUE, if signature is valid, FALSE otherwise
 	 */
-	bool (*verify_signature) (signer_t *this, chunk_t data, chunk_t signature);
+	bool (*verify_signature)(signer_t *this, chunk_t data, chunk_t signature);
 
 	/**
 	 * Get the block size of this signature algorithm.
 	 *
 	 * @return			block size in bytes
 	 */
-	size_t (*get_block_size) (signer_t *this);
+	size_t (*get_block_size)(signer_t *this);
 
 	/**
 	 * Get the key size of the signature algorithm.
 	 *
 	 * @return			key size in bytes
 	 */
-	size_t (*get_key_size) (signer_t *this);
+	size_t (*get_key_size)(signer_t *this);
 
 	/**
 	 * Set the key for this object.
 	 *
 	 * @param key		key to set
+	 * @return			TRUE if key set
 	 */
-	void (*set_key) (signer_t *this, chunk_t key);
+	bool (*set_key)(signer_t *this,
+					chunk_t key) __attribute__((warn_unused_result));
 
 	/**
 	 * Destroys a signer_t object.
 	 */
-	void (*destroy) (signer_t *this);
+	void (*destroy)(signer_t *this);
 };
 
 #endif /** SIGNER_H_ @}*/
