@@ -32,16 +32,30 @@ struct private_pkcs1_plugin_t {
 	pkcs1_plugin_t public;
 };
 
-/**
- * Implementation of pkcs1_plugin_t.pkcs1troy
- */
-static void destroy(private_pkcs1_plugin_t *this)
+METHOD(plugin_t, get_name, char*,
+	private_pkcs1_plugin_t *this)
 {
-	lib->creds->remove_builder(lib->creds,
-							(builder_function_t)pkcs1_public_key_load);
-	lib->creds->remove_builder(lib->creds,
-							(builder_function_t)pkcs1_private_key_load);
+	return "pkcs1";
+}
 
+METHOD(plugin_t, get_features, int,
+	private_pkcs1_plugin_t *this, plugin_feature_t *features[])
+{
+	static plugin_feature_t f[] = {
+		PLUGIN_REGISTER(PRIVKEY, pkcs1_private_key_load, FALSE),
+			PLUGIN_PROVIDE(PRIVKEY, KEY_RSA),
+		PLUGIN_REGISTER(PUBKEY, pkcs1_public_key_load, FALSE),
+			PLUGIN_PROVIDE(PUBKEY, KEY_ANY),
+		PLUGIN_REGISTER(PUBKEY, pkcs1_public_key_load, FALSE),
+			PLUGIN_PROVIDE(PUBKEY, KEY_RSA),
+	};
+	*features = f;
+	return countof(f);
+}
+
+METHOD(plugin_t, destroy, void,
+	private_pkcs1_plugin_t *this)
+{
 	lib->encoding->remove_encoder(lib->encoding, pkcs1_encoder_encode);
 
 	free(this);
@@ -50,18 +64,19 @@ static void destroy(private_pkcs1_plugin_t *this)
 /*
  * see header file
  */
-plugin_t *plugin_create()
+plugin_t *pkcs1_plugin_create()
 {
-	private_pkcs1_plugin_t *this = malloc_thing(private_pkcs1_plugin_t);
+	private_pkcs1_plugin_t *this;
 
-	this->public.plugin.destroy = (void(*)(plugin_t*))destroy;
-
-	lib->creds->add_builder(lib->creds, CRED_PUBLIC_KEY, KEY_ANY,
-							(builder_function_t)pkcs1_public_key_load);
-	lib->creds->add_builder(lib->creds, CRED_PUBLIC_KEY, KEY_RSA,
-							(builder_function_t)pkcs1_public_key_load);
-	lib->creds->add_builder(lib->creds, CRED_PRIVATE_KEY, KEY_RSA,
-							(builder_function_t)pkcs1_private_key_load);
+	INIT(this,
+		.public = {
+			.plugin = {
+				.get_name = _get_name,
+				.get_features = _get_features,
+				.destroy = _destroy,
+			},
+		},
+	);
 
 	lib->encoding->add_encoder(lib->encoding, pkcs1_encoder_encode);
 
