@@ -23,11 +23,39 @@
 
 typedef struct plugin_t plugin_t;
 
+#include <library.h>
+#include <plugins/plugin_feature.h>
+
 /**
  * Interface definition of a plugin.
  */
 struct plugin_t {
 
+	/**
+	 * Get the name of the plugin.
+	 *
+	 * @return		plugin name
+	 */
+	char* (*get_name)(plugin_t *this);
+
+	/**
+	 * Get plugin features with dependencies.
+	 *
+	 * The returned array contains features provided by the plugin and
+	 * dependencies for that feature. See plugin_feature_t for details.
+	 *
+	 * @param features	pointer receiving plugin features
+	 * @return			number of features
+	 */
+	int (*get_features)(plugin_t *this, plugin_feature_t *features[]);
+
+	/**
+	 * Try to reload plugin configuration.
+	 *
+	 * @return		TRUE if reloaded, FALSE if reloading not supporty by plugin
+	 */
+	bool (*reload)(plugin_t *this);
+
 	/**
 	 * Destroy a plugin instance.
 	 */
@@ -36,11 +64,11 @@ struct plugin_t {
 
 
 /**
- * Plugin constructor function definiton.
+ * Plugin constructor function definition.
  *
- * Each plugin has a constructor functions. This function is called on daemon
+ * Each plugin has a constructor function. This function is called on daemon
  * startup to initialize each plugin.
- * The plugin function is named plugin_create().
+ * The plugin function is named plugin_name_plugin_create().
  *
  * @return				plugin_t instance
  */
