@@ -1,4 +1,5 @@
 /*
+ * Copyright (C) 2013 Tobias Brunner
  * Copyright (C) 2007 Martin Willi
  * Hochschule fuer Technik Rapperswil
  *
@@ -18,8 +19,9 @@
 #include <sqlite3.h>
 #include <unistd.h>
 #include <library.h>
-#include <debug.h>
-#include <utils/mutex.h>
+#include <utils/debug.h>
+#include <threading/mutex.h>
+#include <threading/thread_value.h>
 
 typedef struct private_sqlite_database_t private_sqlite_database_t;
 
@@ -39,11 +41,33 @@ struct private_sqlite_database_t {
 	sqlite3 *db;
 
 	/**
-	 * mutex used to lock execute()
+	 * thread-specific transaction, as transaction_t
+	 */
+	thread_value_t *transaction;
+
+	/**
+	 * mutex used to lock execute(), if necessary
 	 */
 	mutex_t *mutex;
 };
 
+/**
+ * Database transaction
+ */
+typedef struct {
+
+	/**
+	 * Refcounter if transaction() is called multiple times
+	 */
+	refcount_t refs;
+
+	/**
+	 * TRUE if transaction was rolled back
+	 */
+	bool rollback;
+
+} transaction_t;
+
 /**
  * Create and run a sqlite stmt using a sql string and args
  */
@@ -110,11 +134,13 @@ static sqlite3_stmt* run(private_sqlite_database_t *this, char *sql,
 	}
 	else
 	{
-		DBG1("preparing sqlite statement failed: %s", sqlite3_errmsg(this->db));
+		DBG1(DBG_LIB, "preparing sqlite statement failed: %s",
+			 sqlite3_errmsg(this->db));
 	}
 	if (res != SQLITE_OK)
 	{
-		DBG1("binding sqlite statement failed: %s", sqlite3_errmsg(this->db));
+		DBG1(DBG_LIB, "binding sqlite statement failed: %s",
+			 sqlite3_errmsg(this->db));
 		sqlite3_finalize(stmt);
 		return NULL;
 	}
@@ -160,7 +186,7 @@ static bool sqlite_enumerator_enumerate(sqlite_enumerator_t *this, ...)
 		case SQLITE_ROW:
 			break;
 		default:
-			DBG1("stepping sqlite statement failed: %s",
+			DBG1(DBG_LIB, "stepping sqlite statement failed: %s",
 				 sqlite3_errmsg(this->database->db));
 			/* fall */
 		case SQLITE_DONE:
@@ -203,7 +229,8 @@ static bool sqlite_enumerator_enumerate(sqlite_enumerator_t *this, ...)
 				break;
 			}
 			default:
-				DBG1("invalid result type supplied");
+				DBG1(DBG_LIB, "invalid result type supplied");
+				va_end(args);
 				return FALSE;
 		}
 	}
@@ -211,10 +238,8 @@ static bool sqlite_enumerator_enumerate(sqlite_enumerator_t *this, ...)
 	return TRUE;
 }
 
-/**
- * Implementation of database_t.query.
- */
-static enumerator_t* query(private_sqlite_database_t *this, char *sql, ...)
+METHOD(database_t, query, enumerator_t*,
+	private_sqlite_database_t *this, char *sql, ...)
 {
 	sqlite3_stmt *stmt;
 	va_list args;
@@ -246,10 +271,8 @@ static enumerator_t* query(private_sqlite_database_t *this, char *sql, ...)
 	return (enumerator_t*)enumerator;
 }
 
-/**
- * Implementation of database_t.execute.
- */
-static int execute(private_sqlite_database_t *this, int *rowid, char *sql, ...)
+METHOD(database_t, execute, int,
+	private_sqlite_database_t *this, int *rowid, char *sql, ...)
 {
 	sqlite3_stmt *stmt;
 	int affected = -1;
@@ -272,7 +295,8 @@ static int execute(private_sqlite_database_t *this, int *rowid, char *sql, ...)
 		}
 		else
 		{
-			DBG1("sqlite execute failed: %s", sqlite3_errmsg(this->db));
+			DBG1(DBG_LIB, "sqlite execute failed: %s",
+				 sqlite3_errmsg(this->db));
 		}
 		sqlite3_finalize(stmt);
 	}
@@ -280,10 +304,81 @@ static int execute(private_sqlite_database_t *this, int *rowid, char *sql, ...)
 	return affected;
 }
 
+METHOD(database_t, transaction, bool,
+	private_sqlite_database_t *this, bool serializable)
+{
+	transaction_t *trans;
+	char *cmd = serializable ? "BEGIN EXCLUSIVE TRANSACTION"
+							 : "BEGIN TRANSACTION";
+
+	trans = this->transaction->get(this->transaction);
+	if (trans)
+	{
+		ref_get(&trans->refs);
+		return TRUE;
+	}
+	if (execute(this, NULL, cmd) == -1)
+	{
+		return FALSE;
+	}
+	INIT(trans,
+		.refs = 1,
+	);
+	this->transaction->set(this->transaction, trans);
+	return TRUE;
+}
+
 /**
- * Implementation of database_t.get_driver
+ * Finalize a transaction depending on the reference count and if it should be
+ * rolled back.
  */
-static db_driver_t get_driver(private_sqlite_database_t *this)
+static bool finalize_transaction(private_sqlite_database_t *this,
+								 bool rollback)
+{
+	transaction_t *trans;
+	char *command = "COMMIT TRANSACTION";
+	bool success;
+
+	trans = this->transaction->get(this->transaction);
+	if (!trans)
+	{
+		DBG1(DBG_LIB, "no database transaction found");
+		return FALSE;
+	}
+
+	if (ref_put(&trans->refs))
+	{
+		if (trans->rollback)
+		{
+			command = "ROLLBACK TRANSACTION";
+		}
+		success = execute(this, NULL, command) != -1;
+
+		this->transaction->set(this->transaction, NULL);
+		free(trans);
+		return success;
+	}
+	else
+	{	/* set flag, can't be unset */
+		trans->rollback |= rollback;
+	}
+	return TRUE;
+}
+
+METHOD(database_t, commit, bool,
+	private_sqlite_database_t *this)
+{
+	return finalize_transaction(this, FALSE);
+}
+
+METHOD(database_t, rollback, bool,
+	private_sqlite_database_t *this)
+{
+	return finalize_transaction(this, TRUE);
+}
+
+METHOD(database_t, get_driver, db_driver_t,
+	private_sqlite_database_t *this)
 {
 	return DB_SQLITE;
 }
@@ -299,12 +394,14 @@ static int busy_handler(private_sqlite_database_t *this, int count)
 	return 1;
 }
 
-/**
- * Implementation of database_t.destroy
- */
-static void destroy(private_sqlite_database_t *this)
+METHOD(database_t, destroy, void,
+	private_sqlite_database_t *this)
 {
-	sqlite3_close(this->db);
+	if (sqlite3_close(this->db) == SQLITE_BUSY)
+	{
+		DBG1(DBG_LIB, "sqlite close failed because database is busy");
+	}
+	this->transaction->destroy(this->transaction);
 	this->mutex->destroy(this->mutex);
 	free(this);
 }
@@ -320,24 +417,31 @@ sqlite_database_t *sqlite_database_create(char *uri)
 	/**
 	 * parse sqlite:///path/to/file.db uri
 	 */
-	if (!strneq(uri, "sqlite://", 9))
+	if (!strpfx(uri, "sqlite://"))
 	{
 		return NULL;
 	}
 	file = uri + 9;
 
-	this = malloc_thing(private_sqlite_database_t);
-
-	this->public.db.query = (enumerator_t* (*)(database_t *this, char *sql, ...))query;
-	this->public.db.execute = (int (*)(database_t *this, int *rowid, char *sql, ...))execute;
-	this->public.db.get_driver = (db_driver_t(*)(database_t*))get_driver;
-	this->public.db.destroy = (void(*)(database_t*))destroy;
-
-	this->mutex = mutex_create(MUTEX_TYPE_RECURSIVE);
+	INIT(this,
+		.public = {
+			.db = {
+				.query = _query,
+				.execute = _execute,
+				.transaction = _transaction,
+				.commit = _commit,
+				.rollback = _rollback,
+				.get_driver = _get_driver,
+				.destroy = _destroy,
+			},
+		},
+		.mutex = mutex_create(MUTEX_TYPE_RECURSIVE),
+		.transaction = thread_value_create(NULL),
+	);
 
 	if (sqlite3_open(file, &this->db) != SQLITE_OK)
 	{
-		DBG1("opening SQLite database '%s' failed: %s",
+		DBG1(DBG_LIB, "opening SQLite database '%s' failed: %s",
 			 file, sqlite3_errmsg(this->db));
 		destroy(this);
 		return NULL;
@@ -347,4 +451,3 @@ sqlite_database_t *sqlite_database_create(char *uri)
 
 	return &this->public;
 }
-
