@@ -31,27 +31,45 @@ struct private_sqlite_plugin_t {
 	sqlite_plugin_t public;
 };
 
-/**
- * Implementation of plugin_t.destroy
- */
-static void destroy(private_sqlite_plugin_t *this)
+METHOD(plugin_t, get_name, char*,
+	private_sqlite_plugin_t *this)
+{
+	return "sqlite";
+}
+
+METHOD(plugin_t, get_features, int,
+	private_sqlite_plugin_t *this, plugin_feature_t *features[])
+{
+	static plugin_feature_t f[] = {
+		PLUGIN_REGISTER(DATABASE, sqlite_database_create),
+			PLUGIN_PROVIDE(DATABASE, DB_SQLITE),
+	};
+	*features = f;
+	return countof(f);
+}
+
+METHOD(plugin_t, destroy, void,
+	private_sqlite_plugin_t *this)
 {
-	lib->db->remove_database(lib->db,
-							 (database_constructor_t)sqlite_database_create);
 	free(this);
 }
 
 /*
  * see header file
  */
-plugin_t *plugin_create()
+plugin_t *sqlite_plugin_create()
 {
-	private_sqlite_plugin_t *this = malloc_thing(private_sqlite_plugin_t);
+	private_sqlite_plugin_t *this;
 
-	this->public.plugin.destroy = (void(*)(plugin_t*))destroy;
-
-	lib->db->add_database(lib->db,
-						  (database_constructor_t)sqlite_database_create);
+	INIT(this,
+		.public = {
+			.plugin = {
+				.get_name = _get_name,
+				.get_features = _get_features,
+				.destroy = _destroy,
+			},
+		},
+	);
 
 	return &this->public.plugin;
 }
