@@ -55,6 +55,14 @@ TEST_VECTOR_CRYPTER(des3_cbc2)
 TEST_VECTOR_CRYPTER(idea1)
 TEST_VECTOR_CRYPTER(idea2)
 TEST_VECTOR_CRYPTER(null1)
+TEST_VECTOR_CRYPTER(rc2_1)
+TEST_VECTOR_CRYPTER(rc2_2)
+TEST_VECTOR_CRYPTER(rc2_3)
+TEST_VECTOR_CRYPTER(rc2_4)
+TEST_VECTOR_CRYPTER(rc2_5)
+TEST_VECTOR_CRYPTER(rc2_6)
+TEST_VECTOR_CRYPTER(rc2_7)
+TEST_VECTOR_CRYPTER(rc2_8)
 TEST_VECTOR_CRYPTER(rc5_1)
 TEST_VECTOR_CRYPTER(rc5_2)
 TEST_VECTOR_CRYPTER(serpent_cbc1)
@@ -140,6 +148,7 @@ TEST_VECTOR_HASHER(md5_7)
 TEST_VECTOR_HASHER(sha1_1)
 TEST_VECTOR_HASHER(sha1_2)
 TEST_VECTOR_HASHER(sha1_3)
+TEST_VECTOR_HASHER(sha1_4)
 TEST_VECTOR_HASHER(sha224_1)
 TEST_VECTOR_HASHER(sha224_2)
 TEST_VECTOR_HASHER(sha224_3)
